/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
q3master@hendriklipka.de
*/

package de.hendriklipka.q3master;

import org.apache.log4j.*;
import de.hendriklipka.q3master.handler.serverquery.*;

import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

public class ProtocolVersionHandlerFactory
{
    private static Category CAT=Category.getInstance(ProtocolVersionHandlerFactory.class.getName());
    private static Category LOG_CAT=Category.getInstance("q3master.protocol");
    
    private static Map _handlers=new HashMap();
    
    static
    {
        _handlers.put(new Integer(43),Protocol43Handler.class.getName());
        _handlers.put(new Integer(45),Protocol43Handler.class.getName());
        _handlers.put(new Integer(48),Protocol48Handler.class.getName());
        _handlers.put(new Integer(57),Protocol48Handler.class.getName());
        _handlers.put(new Integer(58),Protocol48Handler.class.getName());
        _handlers.put(new Integer(59),Protocol48Handler.class.getName());
        _handlers.put(new Integer(60),Protocol48Handler.class.getName());
        _handlers.put(new Integer(66),Protocol48Handler.class.getName());
        _handlers.put(new Integer(67),Protocol48Handler.class.getName());
        _handlers.put(new Integer(68),Protocol48Handler.class.getName());
        _handlers.put(new Integer(82),Protocol48Handler.class.getName());
        _handlers.put(new Integer(83),Protocol48Handler.class.getName());
    }

    public static ProtocolVersionHandler getProtocolVersionHandler(int version)
    {
        Integer v=new Integer(version);
        String className=(String)_handlers.get(v);
        if (null==className)
        {
            LOG_CAT.warn("no handler class for protocol "+version+" configured");
            return null;
        }
        try
        {
            Class c=Class.forName(className);
            return (ProtocolVersionHandler)c.newInstance();
        }
        catch (Exception e)
        {
            LOG_CAT.warn("cannot instantiate handler class for protocol "+version+": "+className);
            return null;
        }
    }
    
    public static void configure(Configuration config)
    {
        Map handlers=config.getServerQueryHandlers();
        if (null!=handlers&&!handlers.isEmpty())
        {
            Iterator prots=handlers.keySet().iterator();
            while (prots.hasNext())
            {
                String prot=(String)prots.next();
                String handler=(String)handlers.get(prot);
                Integer protInt=new Integer(prot);
                _handlers.put(protInt,handler);
            }
        }
    }
}