/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
q3master@hendriklipka.de
*/

package de.hendriklipka.q3master;

/**
 * Title:        Quake3Arena MasterBrowser
 * Description:
 * Copyright:    Copyright (c) 2001 by Hendrik Lipka
 * @author Hendrik Lipka
 * @version 0.1
*/

import java.net.DatagramPacket;
import java.net.DatagramSocket;

import java.io.InterruptedIOException;

import org.apache.log4j.*;

public class RequestListener extends Thread
{
    private Category CAT=Category.getInstance(RequestListener.class.getName());
    private Category LOG_CAT=Category.getInstance("q3master.request");

    protected boolean _stop=false;
    protected int _port=-1;
    protected ServerStateManager _statusHandler=null;

    private final char[] HEXCHARS={'0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F',};

    public RequestListener(int port, ServerStateManager handler)
    {
        super("RequestListener_"+port);
        _port=port;
        _statusHandler=handler;
    }

    public void stopThread()
    {
        _stop=true;
    }

    public void run()
    {
        DatagramSocket serverSocket=null;
        try
        {
//            serverSocket=new DatagramSocket(_port,InetAddress.getByName("127.0.0.1"));
            serverSocket=new DatagramSocket(_port);
            serverSocket.setSoTimeout(5000);
            while (!_stop)
            {
                try
                {
                    byte[] data=new byte[60000];
                    DatagramPacket packet=new DatagramPacket(data,data.length);
                    serverSocket.receive(packet);
                    CAT.debug("got request from "+packet.getAddress()+":"+packet.getPort());
                    dumpPacket(packet);
                    byte[] result=new byte[packet.getLength()];
                    System.arraycopy(packet.getData(),0,result,0,result.length);
                    new RequestHandler(result,packet.getAddress(),packet.getPort(),serverSocket,_statusHandler).start(); // create and start request handler thread
                }
                catch (InterruptedIOException iiex) {} // ignore - this is an expected case...
                catch (Exception e)
                {
                    e.printStackTrace();
                }
            }
            CAT.debug("finished main loop");
        }
        catch (Exception ex)
        {
            ex.printStackTrace();
        }
        finally
        {
//            if (null!=serverSocket)
//                serverSocket.close();
        }
    }
    private void dumpPacket(DatagramPacket packet)
    {
        CAT.debug("dumping packet data");
        byte[] data=packet.getData();
        StringBuffer result=new StringBuffer();
        for (int i=0;i<packet.getLength();i++)
        {
            int c=data[i];
//            dumpCharacter(c,i);
            if (c<32) c=(int)'?';
            result.append((char)c);
        }
        CAT.debug("data="+result.toString());
    }

    private void dumpCharacter(int c, int pos)
    {
        String result=""+pos+": ";
        int ch=c;
        if (ch<32) ch='?';
        result+=((char)ch)+" / ";
        int high=(c>>4)&0xf;
        result+=HEXCHARS[high];
        int low=c&0xf;
        result+=HEXCHARS[low];
        result+=" / "+c;
        CAT.debug(result);
    }
}