/*
Copyright (C) 1999/2000 Hendrik Lipka.
All rights reserved.

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY
WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public License along with this
program; if not, write to the Free Software Foundation, Inc., 675 Mass Ave,
Cambridge, MA 02139, USA or see http://www.gnu.org

If you encounter problems or have a suggestion, please mail to:
q3master@hendriklipka.de
*/

package de.hendriklipka.q3master;

/**
 * Title:        Quake3Arena MasterBrowser
 * Description:
 * Copyright:    Copyright (c) 2001 by Hendrik Lipka
 * @author Hendrik Lipka
 * @version 0.1
*/

import java.util.Hashtable;
import java.util.Enumeration;
import java.util.ArrayList;
import java.text.*;
import java.io.IOException;

import org.apache.log4j.*;

public class ServerStateManager
{
    private Category CAT=Category.getInstance(ServerStateManager.class.getName());
    private Category LOG_CAT=Category.getInstance("q3master.status");

    protected Hashtable _servers=new Hashtable();

    protected long _timeout=1000*60*11; // 11 minutes - heartbeat is sent every 5 minutes

    protected DateFormat _format=DateFormat.getTimeInstance(DateFormat.MEDIUM);

    protected static ServerStatusParser _statusParser=new ServerStatusParser();

    protected Configuration _config=null;

    public void StatusHandler()
    {
        _timeout=Configuration.getInstance().getServerTimeout();
        _config=Configuration.getInstance();
    }

    public void addServer(ServerData data)
    {
        boolean state=_statusParser.queryServerStatus(data);
        if (state)
        {
            _servers.put(data._serverAddress.getHostAddress()+":"+Integer.toString(data._serverPort),data);
            LOG_CAT.info(" - adding server with: "+data._serverAddress.getHostAddress()+":"+Integer.toString(data._serverPort)+"/protocol="+data._protocol);
        }
        else
        {
            _servers.remove(data._serverAddress.getHostAddress()+":"+Integer.toString(data._serverPort));
            LOG_CAT.info(" - removing server with: "+data._serverAddress.getHostAddress()+":"+Integer.toString(data._serverPort));
        }
        CAT.info("server count="+_servers.size());
    }
    
    public void removeServer(ServerData data)
    {
        _servers.remove(data._serverAddress.getHostAddress()+":"+Integer.toString(data._serverPort));
        LOG_CAT.info(" - removing server with: "+data._serverAddress.getHostAddress()+":"+Integer.toString(data._serverPort));
        CAT.info("server count="+_servers.size());
    }

    public void addServerByBroadcast(ServerData data)
    {
        addServer(data);
        // we send this server always to the server, because then it can also be removed
        // if it goes down
        if (null!=Configuration.getInstance().getBroadcastMaster())
        {
            String[] addrs=Configuration.getInstance().getBroadcastMaster();
            for (int i=0;i<addrs.length;i++)
            {
                CAT.debug("sending registration to "+addrs[i]);
                byte[] command=new byte[8+6];
                command[0]='r';
                command[1]='e';
                command[2]='g';
                command[3]='i';
                command[4]='s';
                command[5]='t';
                command[6]='e';
                command[7]='r';
                command[8]=data._serverAddress.getAddress()[0];
                command[9]=data._serverAddress.getAddress()[1];
                command[10]=data._serverAddress.getAddress()[2];
                command[11]=data._serverAddress.getAddress()[3];
                command[12]=(byte)(data._serverPort>>8);
                command[13]=(byte)(data._serverPort&0xff);
                try
                {
                    Util.sendRequest(addrs[i],command,null,27950);
                }
                catch (IOException ex)
                {
                    CAT.error(ex);
                }
            }
        }
    }

    public void addServerByRegistration(ServerData data)
    {
        addServer(data);
    }

    public ServerData[] getAllServers()
    {
        return getAllServers(-1,true,true);
    }

    public ServerData[] getAllServers(int protocol)
    {
        return getAllServers(protocol,true,true);
    }

    public ServerData[] getAllServers(boolean full, boolean empty)
    {
        return getAllServers(-1,full,empty);
    }

    public ServerData[] getAllServers(int protocol, boolean full, boolean empty)
    {
        long time=System.currentTimeMillis();

        Enumeration keys=_servers.keys();
        ArrayList v=new ArrayList();
        while (keys.hasMoreElements())
        {
            Object o=keys.nextElement();
            ServerData sd=(ServerData)_servers.get(o);
            // we need to check for null here, because other threads might remove timed out servers...
            if (null!=sd&&null!=o)
            {
                // check for timeout
                if ((time-sd._lastHeartBeat)>_timeout)
                {
                    LOG_CAT.info("removing server with key "+(String)o+" because of timeout since last heartbeat");
                    _servers.remove(o);
                    continue;
                }
                // filter
                CAT.debug("server="+sd._serverAddress+":"+sd._serverPort);
                _config=Configuration.getInstance();
                if (!full&&(sd._players==sd._maxPlayers)&&_config.filterFull())
                {
                    CAT.debug("filtered because full");
                    continue;
                }
                if (!empty&&(0==sd._players)&&_config.filterEmpty())
                {
                    CAT.debug("filtered because empty");
                    continue;
                }
                if ((-1!=protocol)&&(protocol!=sd._protocol)&&_config.filterProtocol())
                {
                    CAT.debug("filtered because wrong protocol");
                    continue;
                }
                // add server to result
                CAT.debug("added server <"+sd._serverAddress+":"+sd._serverPort+"> to result");
                v.add(sd);
            }
        }

        if (0==v.size())
            return null;

        ServerData[] sda=new ServerData[1];
        sda=(ServerData[])v.toArray(sda);
        
//        ServerData[] sda=new ServerData[v.size()];
//        for (int i=0;i<sda.length;i++)
//        {
//            sda[i]=(ServerData)v.elementAt(i);
//        }
        return sda;
    }
}